/**
 * Main Muck Output Window
 * $Id: MuckMain.java,v 1.59 2004/08/03 02:06:40 jeffnik Exp $
 */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2005  Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package anecho.JamochaMUD;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
// import java.awt.event.ActionListener;
import java.net.*;
import java.io.*;
import java.util.*;

import anecho.gui.SyncFrame;
import anecho.gui.JMText;
import anecho.gui.ResReader;
import anecho.gui.PosTools;

import anecho.JamochaMUD.plugins.PlugInterface;

/**
 * Main Muck Output Window, heart of the program, containing
 * most of the menus and logistics of JamochaMUD
 * @version $Id: MuckMain.java,v 1.59 2004/08/03 02:06:40 jeffnik Exp $
 * @author Jeff Robinson
 */
// public class MuckMain extends SyncFrame implements ActionListener, ComponentListener, KeyListener, ItemListener, WindowListener{
public class MuckMain implements ActionListener, ComponentListener, KeyListener, ItemListener, WindowListener {
    
    private CheckboxMenuItem splitFramesItem;
    private CheckboxMenuItem tWAltFocus;
    
    static boolean pauseStatus, textWindowStatus;
    static BufferedReader input;
    private Frame muckMainFrame;
    static Socket serverSock;
    // private String muckMainTitle;
    // private Vector tempVector;
    protected static Menu tWPlugInMenu;
    public static final int COMBINED = 0;     // Combined frame style
    public static final int SPLIT = 1;        // Split frame style
    private int viewStyle = COMBINED;         // the current style of our display
    private GridBagLayout mainBagLayout;
    private GridBagConstraints constraints;
    // private CardLayout textCardLayout;              // The layout manage that contains the MU* output
    // private Panel textPanel;                  // A panel to hold our text-area (for use with the layout managers)
    private JMConfig settings;                // Our settings
    private static final int NEXT = 1;
    private static final int PREVIOUS = 0;
    private static ResReader reader;
    private boolean listenerSet = false;        // Have we set the listener for JTabbedPane if necessary
    private boolean useSwing = true;   // Whether the interface is swing-based or not
    private boolean useSwingMenu = false;       // Right now we have to use the heavyweight AWT menu bar
    
    private static final boolean DEBUG = false;
    
    public MuckMain(JMConfig mainSettings) {
        // super(RB("MuckMain.title"));
        if (mainSettings.getJMboolean(JMConfig.USESWING)) {
            // muckMainFrame = (Frame)new anecho.gui.JSyncFrame("JamochaMUD");
            muckMainFrame = (Frame)new anecho.JamochaMUD.MainSwingFrame("JamochaMUD");
            useSwing = true;
            useSwingMenu = true;
        } else {
            useSwing = false;
            useSwingMenu = false;
            muckMainFrame = (Frame)new SyncFrame("JamochaMUD");
        }
        
        this.settings = mainSettings;
        settings.setJMObject(JMConfig.MUCKMAINFRAME, muckMainFrame);
        
        reader = new ResReader();       // Initialise our ResReader to handle our language issues
        
        // Create the windows
        // muckMainTitle = new String(RB("MuckMain.title"));
        
        muckMainFrame.addComponentListener(this);
        muckMainFrame.addWindowListener(this);
        muckMainFrame.addKeyListener(this);
        
        pauseStatus = false;
        //Vector tempVector = new Vector(0, 1);
        
        // Add Menu Bar and items to textWindow
        // MenuBar tWMenuBar = new MenuBar();
        
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            swingMenu.buildMenu((javax.swing.JFrame)muckMainFrame, this, settings);
        } else {
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            mainMenu.buildMenu(muckMainFrame, this, settings);
        }
        
        MediaTracker tracker = new MediaTracker(muckMainFrame);
        // Image jamochaMUDImage = Toolkit.getDefaultToolkit().getImage("kehza.gif");
        Class testC = this.getClass();
        Image jMUDImage = Toolkit.getDefaultToolkit().getImage(testC.getResource("kehza.gif"));
        tracker.addImage(jMUDImage, 0);
        try {
            tracker.waitForAll();
        } catch (Exception except) {
            if (DEBUG) {
                System.err.println("Icon load error.  Exception " + except);
            }
        }
        muckMainFrame.setIconImage(jMUDImage);
    }
    
    /**
     * This is a generic bit to access the
     * ResReader.class, for localization
     * (Multi-language support)
     */
    private String RB(String itemTarget) {
        // ResReader reader = new ResReader();
        return reader.LangString(JMConfig.BUNDLEBASE, itemTarget);
    }
    
    // This supports multi-line messages
    private static Vector RBL(String itemTarget) {
        // ResReader reader = new ResReader();
        return reader.LangVector(JMConfig.BUNDLEBASE, itemTarget);
    }
    
    private static String[] RBA(String itemTarget) {
        return reader.langArray(JMConfig.BUNDLEBASE, itemTarget);
    }
    
    public void actionPerformed(ActionEvent event){
        
        String arg = event.getActionCommand();
        // System.err.println("Event: " + event);
        
        if (arg.equals(RB("nextMU"))) {
            advanceMU(NEXT);
        }
        
        if (arg.equals(RB("previousMU"))) {
            advanceMU(PREVIOUS);
        }
        
        // Arguments for the ***FILE**** menu
        
        // Check for 'Dump Output'
        // if (arg.equals(dumpOutputItem.getLabel())) {
        if (arg.equals(RB("dumpOutput"))) {
            dumpOutput();
        }
        
        if (arg.equals(RB("quit"))) {
            // Do the proper exit proceedure
            quitJamochaMUD();
            
            // Close the JVM
            System.exit(1);  // Probably an ugly exit
        }
        
        // Arguments for the **EDIT** menu
        
        /** Copy */
        if (arg.equals(RB("copyFromMainWindow"))) {
            copyToClipboard();
        }
        
        /** Paste */
        if (arg.equals(RB("paste"))) {
            pasteFromClipboard();
        }
        
        /** Find */
        
        // Arguments for the **CONNECTION**** menu
        
        /** Connect to MU* */
        if (arg.equals(RB("connectToMU"))) {
            jmConnectToMU();
        }
        
        /** Reconnect to MU* */
        if (arg.equals(RB("reconnectToMU"))) {
            jmReconnectToMU();
        }
        
        /** Disconnect from MU* */
        if (arg.equals(RB("disconnectFromMU"))) {
            jmDisconnectFromMU();
        }
        
        /** Close the visible MU "window" */
        if (arg.equals(RB("closeThisView"))) {
            closeActiveWindow();
        }
        
        /** Ping MU* */
        // if (arg.equals(pingMUItem.getLabel())) {
        //
        // }
        
        // Arguments for ****OPTIONS**** menu
        
        if (arg.equals(RB("fontsAndColours"))) {
            setColours();
        }
        
        if (arg.equals(RB("externalPrograms"))) {
            showExtProgDialogue();
        }
        
        if (arg.equals(RB("serverOptions"))) {
            showProxyDialogue();
        }
        
        if (arg.equals(RB("ManagePlugins.title"))) {
            showPluginsDialogue();
            // return;
        }
        
        if (arg.equals(RB("installPlugin"))) {
            installPlugIn();
        }
        
        // Arguments for ****HELP**** menu
        if (arg.equals(RB("help"))) {
            StringBuffer tentativeURL = new StringBuffer(RB("jamochaMUDPages"));
            
            // Attempt to use BrowserLauncher!
            try {
                String browser = settings.getJMString(JMConfig.BROWSER1);
                String url = tentativeURL.toString();
                edu.stanford.ejalbert.BrowserLauncher.openURL(browser, url);
            } catch (Exception except) {
                // The was an error trying to launch the browser
                // if (settings.getJMboolean(JMConfig.USESWING)) {
                if (useSwing) {
                    javax.swing.JOptionPane.showMessageDialog(muckMainFrame,
                            RBA("configureBrowser"),
                            RB("configureExternalProgram"),
                            javax.swing.JOptionPane.ERROR_MESSAGE);
                } else {
                    Vector tempVector = new Vector(0, 1);
                    anecho.gui.OKBox check = new anecho.gui.OKBox(muckMainFrame, RB("configureExternalProgram"), true);
                    tempVector = RBL("configureBrowser");
                    
                    for (int i = 0; i < tempVector.size(); i++) {
                        check.append((String)tempVector.elementAt(i));
                    }
                    
                    check.showCentered();
                }
            }
        }
        
        if (arg.equals(RB("tinyFugueKeyCodes"))) {
            JMTFKeys tfKeys = JMTFKeys.getInstance();        // Trying out the JMTFKeys singleton
            // JMTFKeys.showCommands(this);
            // tfKeys.showCommands(muckMainFrame);
            tfKeys.showCommands(muckMainFrame, useSwing);
        }
        
        if (arg.equals(RB("aboutJamochaMUD"))) {
            showAboutBox();
        }
        
        // if (arg.equals(RB("troubleshooting"))) {
        if (arg.equals(RB("reportABug"))) {
            // if (settings.getJMboolean(JMConfig.USESWING)) {
            
            String message = anecho.gui.AbstractMessageFormat.wrap(RB("reportABugMessage"));
            
            if (useSwing) {
                /*
                javax.swing.JOptionPane.showMessageDialog(muckMainFrame,
                                                          RBA("reportABugMessage"),
                                                          RB("reportABug"),
                                                          javax.swing.JOptionPane.INFORMATION_MESSAGE);
                 */
                javax.swing.JOptionPane.showMessageDialog(muckMainFrame,
                        message,
                        RB("reportABug"),
                        javax.swing.JOptionPane.INFORMATION_MESSAGE);
            } else {
                // anecho.gui.OKBox shoot = new anecho.gui.OKBox(this, RB("reportABug"), true);
                anecho.gui.OKBox shoot = new anecho.gui.OKBox(muckMainFrame, RB("reportABug"), true);
                // anecho.gui.OKBox shoot = new anecho.gui.OKBox(muckMainFrame, message, true);
                /*
                Vector mess = RBL("reportABugMessage");
                for (int i = 0; i < mess.size(); i++) {
                    shoot.append((String)mess.elementAt(i));
                }
                 */
                shoot.append(message);
                
                shoot.show();
            }
        }
        
        // Check to see if we've had a request on the plugins menu
        // if (event.getSource() == tWPlugInMenu) {
        if (arg.startsWith("plugin:") || arg.startsWith("enable:") || arg.startsWith("disable:") || arg.startsWith("description")) {
            
            int col = arg.indexOf(':');
            
            // String testName = arg.substring(7);
            String testName = arg.substring(col + 1);
            
            // System.err.println("Request on tWPlugInMenu");
            // a request for the properties of a plugin has been made
            try {
                // int selected = EnumPlugIns.plugInName.indexOf(event.getActionCommand());
                int selected = EnumPlugIns.plugInName.indexOf(testName);
                Object plugClass = EnumPlugIns.plugInClass.elementAt(selected);
                String name = ((PlugInterface)plugClass).PlugInName();
                
                if (arg.startsWith("plugin:")) {
                    ((PlugInterface)plugClass).PlugInProperties();
                }
                
                if (arg.startsWith("description:")) {
                    if (DEBUG) {
                        System.err.println("Calling description");
                    }
                    
                    String desc = anecho.gui.AbstractMessageFormat.wrap(((PlugInterface)plugClass).PlugInDescription());
                    
                    if (useSwing) {
                        javax.swing.JOptionPane.showMessageDialog(muckMainFrame, desc, name, javax.swing.JOptionPane.INFORMATION_MESSAGE);
                    } else {
                        anecho.gui.OKBox message = new anecho.gui.OKBox(muckMainFrame, desc);
                        message.setTitle(name);
                        message.showCentered();
                    }
                }
                
                if (arg.startsWith("enable:")) {
                    if (DEBUG) {
                        System.err.println("Calling enable");
                    }
                    
                    ManagePlugins manplug = new ManagePlugins(settings);
                    manplug.changePlugInState(name, true);
                    
                    // Update the menu to reflect our new state
                    // rebuildPlugInMenu();
                }
                
                if (arg.startsWith("disable:")) {
                    if (DEBUG) {
                        System.err.println("Calling disable");
                    }
                    
                    ManagePlugins manplug = new ManagePlugins(settings);
                    manplug.changePlugInState(name, false);
                    
                    // Update the menu to reflect our new state
                    // rebuildPlugInMenu();
                }
                
            } catch (ArrayIndexOutOfBoundsException except) {
                // System.err.println("Plug-in menu exception: no settings available.");
                // anecho.gui.OKBox message = new anecho.gui.OKBox(this, RB("noSettingsAvailable"), true, RB("noPlugInSettings"));
                if (DEBUG) {
                    System.err.println("Error trying to get plug-in options.");
                    System.err.println(except);
                    except.printStackTrace();
                }
                
                if (useSwing) {
                    javax.swing.JOptionPane.showMessageDialog(muckMainFrame, RB("noSettingsAvailable"));
                } else {
                    anecho.gui.OKBox message = new anecho.gui.OKBox(muckMainFrame, RB("noSettingsAvailable"), true, RB("noPlugInSettings"));
                    message.showCentered();
                }
            }
        }
        
        // if (event.getSource() == tWMUListMenu) {
        if (event.getActionCommand().startsWith("ChangeMU:")) {
            // Make sure we tell the ConnHandler we're changing MU*s
            setActiveMU(event.getActionCommand());
            
            // This will show our active MU*
            setVisibleMU();
            
            // Update our connection menu
            updateConnectionMenu();
            
            // Update the Frame's title
            setWindowTitle();
            
            // Make sure the info shows up!
            // validate();
            muckMainFrame.validate();
        }
        
    }
    
    /**
     * Show a dialogue box to allow users to select a file
     * to dump all of the output to for logging/archiving, etc...
     */
    private void dumpOutput() {
        
        // Show the system dump dialogue
        // This will dump *all* the output to the selected file
        // String prevPath = settings.getLogPath();
        String prevPath = settings.getJMString(settings.LOGPATH);
        
        // try {
        // FileDialog dumpDialogue = new FileDialog(this, "Dump Output to file", 1);
        FileDialog dumpDialogue = new FileDialog(muckMainFrame, "Dump Output to file", 1);
        if (prevPath != null && !prevPath.equals("")) {
            dumpDialogue.setDirectory(prevPath);
        }
        dumpDialogue.show();
        
        try {
            String fileName = dumpDialogue.getFile();
            
            // File has been selected
            // now write the output contents to it
            // try {
            if (fileName != null) {
                String filePath = dumpDialogue.getDirectory();
                FileOutputStream outFile = new FileOutputStream((filePath + fileName), false);
                PrintWriter out = new PrintWriter(outFile, true);
                CHandler connHandler = settings.getConnectionHandler();
                if (useSwing) {
                    anecho.gui.JMSwingText activeMU = connHandler.getActiveMUDSwingText();
                    out.println(activeMU.getText());
                } else {
                    JMText activeMU = connHandler.getActiveMUDText();
                    out.println(activeMU.getText());
                }
                out.flush();
                outFile.close();
                settings.setJMValue(settings.LOGPATH, filePath);
            }
            // } catch (Exception fo) {
            // System.err.println("File output error " + fo);
            // }
        } catch (Exception except) {
            // This occurs when there is a 'cancel' instead of a file selection
            // We'll just gracefully fall through
            dumpDialogue.dispose();
        }
        
        // } catch (Exception e) {
        // This occurs if the user hits 'cancel'
        // so we'll just end this sequence.
        // System.err.println("MuckMain, final exception");
        // }
        
    }
    
    public void keyPressed(KeyEvent event){
        // int arg = event.getKeyCode();
        boolean autoFocus = false;
        
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            autoFocus = swingMenu.isAutoFocus();
        } else {
            // muckMainFrame.setMenuBar(tWMenuBar);
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            autoFocus = mainMenu.isAutoFocus();
        }
        
        if (!event.isAltDown() && !event.isControlDown()) {
            // Check to see if the user wants AutoFocus shift
            // if (!tWAutoFocus.getState()) {
            if (autoFocus) {
                transferFocus(event);
            } else {
                event.consume();
            }
        }
    }
    
    public void keyTyped(KeyEvent event){}
    
    public void keyReleased(KeyEvent event){}
    
    public void itemStateChanged(ItemEvent event) {
        String arg = "";
        boolean itemState = false;
        
        if (useSwingMenu) {
            // arg = (event.getItem()).getText();
            Object target = event.getItem();
            arg = ((javax.swing.JCheckBoxMenuItem)target).getText();
        } else {
            arg = (String) event.getItem();
        }
        
        // Determine if our item is now enabled or disabled
        if (event.getStateChange() == ItemEvent.SELECTED) {
            itemState = true;
        }
        
        if (arg.equals(RB("autoFocusInput"))) {
            // settings.setAutoFocusInput(tWAutoFocus.getState());
            // settings.setJMValue(settings.AUTOFOCUSINPUT, tWAutoFocus.getState());
            settings.setJMValue(settings.AUTOFOCUSINPUT, itemState);
        }
        
        if (arg.equals(RB("autoLogging"))) {
            settings.setJMValue(settings.AUTOLOGGING, itemState);
            
            // Tell our different MU*s to start/stop logging
            CHandler connHandler = settings.getConnectionHandler();
            connHandler.setLogging(itemState);
        }
        
        if (arg.equals(RB("altFocus"))) {
            // settings.setAltFocus(tWAltFocus.getState());
            // settings.setJMValue(JMConfig.ALTFOCUS, tWAltFocus.getState());
            settings.setJMValue(JMConfig.ALTFOCUS, itemState);
        }
        
        /* We current have no MacroBar feature in this version of JamochaMUD */
        /*
        if (arg.equals(RB("showMacroBar"))) {
            settings.setMacroVisible(tWMacro.getState());
            // Hide/Show the macro bars
            // FIx this XXX
            // MuckConn.settings.setMacroVisible(tWMacro.getState());
            // MuckConn.jmMacros.setVisible(tWMacro.getState());
            if (tWMacro.getState()) {
                // Make the frame visible
                // MuckConn.jmMacros.setActiveState(true);
         
                if (tWSyncWindowsItem.getState()) {
                    // Fix this XXX
                    // MuckConn.jmMacros.setSync(true);
                } else {
                    // Fix this XXX
                    // MuckConn.jmMacros.setSync(false);
                }
            }
            // else {
            // // Try and call the subroutine from MuMacros
            //    MuckConn.jmMacros.setActiveState(false);
            //    MuckConn.jmMacros.setVisible(false);
            //}
            }
         */
        
        if (arg.equals(RB("localEcho"))) {
            // settings.setLocalEcho(tWLocalEchoItem.getState());
            // settings.setJMValue(JMConfig.LOCALECHO, tWLocalEchoItem.getState());
            settings.setJMValue(JMConfig.LOCALECHO, itemState);
        }
        
        if (arg.equals(RB("doubleBuffer"))) {
            // setDoubleBuffer(tWDoubleBufferItem.getState());
            setDoubleBuffer(itemState);
        }
        
        if (arg.equals(RB("antiAlias"))) {
            setAntiAliasing(itemState);
        }
        
        if (arg.equals(RB("Low_colour_display"))) {
            setLowColour(itemState);
        }
        
        if (arg.equals(RB("splitFrames"))) {
            // settings.setSplitView(splitFramesItem.getState());
            // settings.setJMValue(JMConfig.SPLITVIEW, splitFramesItem.getState());
            settings.setJMValue(JMConfig.SPLITVIEW, itemState);
            // Toggle our layout
            if (viewStyle == SPLIT) {
                viewStyle = COMBINED;
                splitFramesItem.setState(false);
                tWAltFocus.setEnabled(false);
            } else {
                viewStyle = SPLIT;
                splitFramesItem.setState(true);
                tWAltFocus.setEnabled(true);
            }
            
            // Call for new layout
            setMainLayout();
        }
        
        if (arg.equals(RB("syncWindows"))) {
            // settings.setJMValue(JMConfig.SYNCWINDOWS, tWSyncWindowsItem.getState());
            // setAllSync(tWSyncWindowsItem.getState());
            
            // If the user is not using split frames then we can record this choice,
            // but we should not affect the true nature of the frames
            settings.setJMValue(JMConfig.SYNCWINDOWS, itemState);
            
            if (splitFramesItem.getState()) {
                // The frames are split, so we can go to town!
                setAllSync(itemState);
            }
            
        }
        
        if (arg.equals(RB("showTimers"))) {
            // settings.setTimersVisible(tWTimers.getState());
            // settings.setJMValue(JMConfig.TIMERSVISIBLE, tWTimers.getState());
            settings.setJMValue(JMConfig.TIMERSVISIBLE, itemState);
            // Changed the status of the timers
            // Fix this XXX
            // if (tWTimers.getState()) {
            // The timers have been enabled, start the timer thread
            // timerThread.setActiveState(true);
            // } else {
            // timerThread.setActiveState(false);
            // }
        }
        
        if (arg.equals(RB("useUnicode"))) {
            // Changed the Unicode option
            // settings.setJMValue(JMConfig.USEUNICODE, tWUseUnicodeItem.getState());
            settings.setJMValue(JMConfig.USEUNICODE, itemState);
        }
        
        if (arg.equals(RB("releasePause"))) {
            // settings.setReleasePause(tWReleasePauseItem.getState());
            // settings.setJMValue(JMConfig.RELEASEPAUSE, tWReleasePauseItem.getState());
            settings.setJMValue(JMConfig.RELEASEPAUSE, itemState);
        }
        
        if (arg.equals(RB("tinyFugueKeys"))) {
            // settings.setTFKeyEmu(tWTFKeysItem.getState());
            // settings.setJMValue(settings.TFKEYEMU, tWTFKeysItem.getState());
            settings.setJMValue(settings.TFKEYEMU, itemState);
        }
    }
    
    
    
    public void windowActivated(WindowEvent event) {
        // Restore the original title to the window
        // settings.setMainWindowIconified(false);
        settings.setJMValue(JMConfig.MAINWINDOWICONIFIED, false);
        setWindowTitle();
    }
    
    public void windowClosed(WindowEvent event) {
    }
    
    public void windowClosing(WindowEvent event) {
        // Do a proper shutdown proceedure
        quitJamochaMUD();
    }
    
    public void windowDeactivated(WindowEvent event) {}
    
    public void windowDeiconified(WindowEvent event) {
        // System.err.println("Window deiconified.");
        // Just as a test
        /*
        CHandler target = settings.getConnectionHandler();
        MuSocket mSock = target.getActiveMUHandle();
        if (mSock.isPaused()) {
            // System.err.println("Window deiconified shows mSock is paused...");
            // This spools out any paused text
            mSock.SpoolText();
            }
         */
        callSpoolText();
        
        // The text window is visible
        textWindowStatus = true;
        // iconified = false;
        // settings.setMainWindowIconified(false);
        settings.setJMValue(JMConfig.MAINWINDOWICONIFIED, false);
        setWindowTitle();
    }
    
    public void windowIconified(WindowEvent event) {
        // Write the frame to a the Hashtable
        // settings.setMainWindowIconified(true);
        settings.setJMValue(JMConfig.MAINWINDOWICONIFIED, true);
        // settings.setMainWindow(this.getBounds());
        // settings.setJMValue(JMConfig.MAINWINDOW, this.getBounds());
        settings.setJMValue(JMConfig.MAINWINDOW, muckMainFrame.getBounds());
        if (DEBUG) {
            System.err.println("MuckMain.windowIconified set bounds to: " + muckMainFrame.getBounds());
        }
        
    }
    
    public void windowOpened(WindowEvent event) {}
    
    // We'll track the movements of our window for when we have to
    // write it out to our .rc file  Each window for himself!!
    public void componentHidden(ComponentEvent event) {
    }
    
    public void componentResized(ComponentEvent event) {
        // Do we need anything in this method?  I blocked it out!  XXX
        // settings.setJMValue(JMConfig.MAINWINDOW, muckMainFrame.getBounds());
        // System.err.println("MuckMain.componentResized settings bounds to: " + muckMainFrame.getBounds());
    }
    
    public void componentMoved(ComponentEvent event) {
        // settings.setMainWindow(this.getBounds());
        // settings.setJMValue(JMConfig.MAINWINDOW, this.getBounds());
        settings.setJMValue(JMConfig.MAINWINDOW, muckMainFrame.getBounds());
    }
    
    public void componentShown(ComponentEvent event) {
    }
    
    public synchronized void setWindowTitle() {
        // This resets the window's title, depending if the
        // client is connected to a MU* or not
        if (DEBUG) {
            System.err.println("MuckMain.setWindowTitle starts");
        }
        
        CHandler connection = settings.getConnectionHandler();
        if (DEBUG) {
            System.err.println("MuckMain.setWindowTitle checking for active connection");
        }
        
        boolean active = connection.isActiveMUDConnected();
        
        if (DEBUG) {
            System.err.println("MuckMain.setWindowTitle got connection status");
        }
        
        if (active) {
            // Now test to see if this MU* output is paused or not
            // System.err.println("MuckMain.setWindowTitle calls connection.getActiveMUHandle");
            MuSocket mSock = connection.getActiveMUHandle();
            
            if (mSock.isPaused()) {
                // This format is nice, but too long
                // this.setTitle(RB("outputPaused"));
                muckMainFrame.setTitle(RB("outputPaused"));
            } else {
                // this.setTitle(connection.getActiveTitle());
                muckMainFrame.setTitle(connection.getActiveTitle());
            }
        } else {
            // Connection is inactive, just have program's title
            //this.setTitle("JamochaMUD - Not connected");
            muckMainFrame.setTitle("JamochaMUD - Not connected");
        }
        
        if (DEBUG) {
            System.err.println("MuckMain.setWindowTitle going to checkCloseMenuState");
        }
        
        checkCloseMenuState();
        
        if (DEBUG) {
            System.err.println("MuckMain.setWindowTitle finished.");
        }
    }
    
    /** A specific instance of setting the MU* title if
     * the MU* is minimised
     */
    public synchronized void setWindowTitle(String title) {
        // this.setTitle(title);
        muckMainFrame.setTitle(title);
    }
    
    
    /**
     * Methods called when disconnected from the MUD/MUCK
     * Here, we reset some of the menu flags to the appropriate status
     * and give the user visual notification that they have been disconnected
     * (This is not always called just by the user's 'Disconnect' action)
     */
    public void disconnectMenu() {
        // This changes the flags on the MuckMain menu to 'disconnected'
        // Connect to MU*
        /*
        cTM.setEnabled(true);
        // Disconnect from MU*
        dFM.setEnabled(false);
        // Reconnect to MU*
        rTM.setEnabled(true);
         */
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            swingMenu.setConnected(false);
        } else {
            // muckMainFrame.setMenuBar(tWMenuBar);
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            mainMenu.setConnected(false);
        }
    }
    
    /**
     * We've received notification from one of our connections that it
     * has either terminated.  We'll query the mu* and see if it is
     * the active one.  If so, we'll update our connection menu.
     */
    public void checkDisconnectMenu() {
        CHandler connHandler = settings.getConnectionHandler();
        // MuSocket mu = connHandler.getActiveMUHandle();
        
        // If our active MU* is not connected, change the connection menu
        if (!connHandler.isActiveMUDConnected()) {
            disconnectMenu();
        }
    }
    
    /**
     * The user has chosen to connect to a MUD/MUCK, so we
     * set the menu items as appropriate, and give the a visual
     * identifier that we are actually trying to make the connection
     */
    public void connectMenu() {
        /*
        // This changes the flags on the MuckMain menu to 'connected'
        // Connect to MU*
        cTM.setEnabled(true);
        // Disconnect from MU*
        dFM.setEnabled(true);
        // Reconnect to MU*
        rTM.setEnabled(false);
         */
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            swingMenu.setConnected(true);
        } else {
            // muckMainFrame.setMenuBar(tWMenuBar);
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            mainMenu.setConnected(true);
        }
        
    }
    
    /**
     * Set the 'pause' status on the Main window.
     * Usually done by a single mouse-click, it will stop the
     * text from scrolling, and set the DataBar's title to alert
     * the user that the text is paused
     */
    public static void pauseText() {
        // A single click signals a pause
        // This will pause the output window,
        // the incoming lines held in queue
        pauseStatus = true;
    }
    
    public static void refreshPlugInListener() {
        // First, remove the old listener
        // tWPlugInMenu.removeActionListener();
        
        // Now add the Listener to the new list
        // tWPlugInMenu.addActionListener();
    }
    
    /**
     * The user has chosen to connect to a MU*, so we hide all the
     * active windows and show the MuckConn again
     */
    private void jmConnectToMU() {
        CHandler connHandler = settings.getConnectionHandler();
        if (DEBUG) {
            System.err.println("MuckMain.JMConnectToMU calling connHandler.connectToNewMU()");
        }
        
        connHandler.connectToNewMU();
        
        if (DEBUG) {
            System.err.println("MuckMain.JMConnectToMU complete.");
        }
    }
    
    /**
     * Disconnect from the MU, closing the socket and stopping
     * the thread that listens to that IP address
     */
    private void jmDisconnectFromMU() {
        CHandler connHandler = settings.getConnectionHandler();
        // System.err.println("MuckMain.JMDisconnectFromMU calls connHandler.getActiveMUHandle.");
        MuSocket muSock = connHandler.getActiveMUHandle();
        String stamp = muSock.getTimeStamp() + "";
        connHandler.closeActiveMU(stamp);
        
        // Explicitly set the state of our menus
        setConnectionMenu();
        
        // checkCloseMenuState();       // does 'setConnectionMenu()' make this redundant?
    }
    
    /** Reconnect to the MU* we were just connected to! */
    private synchronized void jmReconnectToMU() {
        // We'll remove the current MU* after we grab the name and address
        // It's important to recycle
        CHandler connHandler = settings.getConnectionHandler();
        
        if (DEBUG) {
            // System.err.println("MuckMain.JMReconnectToMU calls .getActiveMUHandle");
            System.err.println("MuckMain.JMReconnectToMU calls CHandler.reconnectToMU()");
        }
        
        connHandler.reconnectToMU();
        
        if (DEBUG) {
            System.err.println("MuckMain.JMReconnectToMU complete");
        }
        
        // Reset titles and stuff
        updateStates();
        
        /*
        MuSocket handle = connHandler.getActiveMUHandle();
        String name = handle.getMUName();
        String address = handle.getAddress();
        int port = handle.getPort();
         
        if (DEBUG) {
            System.err.println("Trying to reconnect to: " + name + " " + address + " " + port);
        }
         
        MuSocket msock = connHandler.OpenSocket(name, address, port);
         
        if (DEBUG) {
            System.err.println("Adding the new MU*");
        }
         
        connHandler.addNewMU(name, address, port, msock);
         
        if (DEBUG) {
            System.err.println("MuckMain.JMReconnectToMU completed adding the new MU*");
        }
         
        if (DEBUG) {
            System.err.println("setVisibleMU()");
        }
         
        setVisibleMU();
         
        if (DEBUG) {
            System.err.println("Closing previous window.");
        }
        closeWindow(handle);         // This causes layout problems?!
         
        if (DEBUG) {
            System.err.println("Completed reconnection method.");
        }
         */
    }
    
    /** Change the shown MU* to our new selection */
    public synchronized void setVisibleMU() {
        // Check to make certain there are MU*'s to show
        // CHandler connHandler = settings.getConnectionHandler();
        
        /*
        if (connHandler.totalConnections() < 1) {
            // There are no windows left to show.
            System.err.println("No connections left to show.");
            return;
        }
         
        // setup the cardLayout to show the proper MU*
        // System.err.println("MuckMain.setVisibleMU calls getActiveMUHandle.");
        MuSocket sock = connHandler.getActiveMUHandle();
        String stamp = sock.getTimeStamp() + "";
         
        // If we don't validate on some OSes our
        // text area won't show up until the frame
        // is resized
        textCardLayout.show(textPanel, stamp);
        // textPanel.invalidate();
        // sock.validate();
        textPanel.validate();
        this.doLayout();
        // textPanel.doLayout();
         */
        // connHandler.setVisibleMU();
        
        // Update the menu to show the appropriate MU* as being active
        updateConnectionMenu();
        
        // Make certain that our connection menu is up to date for this MU*
        setConnectionMenu();
        
        
        // Change the title on our data-entry window
        DataIn inWindow = settings.getDataInVariable();
        inWindow.setWindowTitle();
        // textPanel.invalidate();
    }
    
    /** Set the colouring for our components */
    private void setColours() {
        // Grab our settings for the current MU*
        Color fgColor = settings.getJMColor(settings.FOREGROUNDCOLOUR);
        Color bgColor = settings.getJMColor(settings.BACKGROUNDCOLOUR);
        Font ourFont = settings.getJMFont(settings.FONTFACE);
        
        // Set up our dialogue and display it
        FontFace fFace = new FontFace(settings, muckMainFrame, fgColor, bgColor, ourFont);
        //        fFace.setSize(300, 250);
        fFace.setLocation(anecho.gui.PosTools.findCenter(muckMainFrame, fFace));
        fFace.setResizable(false);
        fFace.setVisible(true);
        // Now querry our dialogue and get our new settings
        Font newStyle = fFace.getFontStyle();
        if (newStyle == null) {
            // The user cancelled, no changes made
            return;
        }
        
        fgColor = fFace.getForegroundColour();
        bgColor = fFace.getBackgroundColour();
        
        // Set up the other components with the proper colour
        // Change the colour/style in our settings
        // settings.setFontFace(newStyle);
        settings.setJMValue(settings.FONTFACE, newStyle);
        // settings.setForegroundColour(fgColor);
        // settings.setBackgroundColour(bgColor);
        settings.setJMValue(settings.FOREGROUNDCOLOUR, fgColor);
        settings.setJMValue(settings.BACKGROUNDCOLOUR, bgColor);
        
        // Change the colours/styles of our input window
        JMUD core = settings.getJMCore();
        core.setAllFonts(newStyle);
        core.setAllColours(fgColor, bgColor);
    }
    
    private void showAboutBox() {
        if (settings.getJMboolean(JMConfig.USESWING)) {
            AboutBox2 temp = new AboutBox2(muckMainFrame, settings);
            temp.setVisible(true);
        } else {
            // AboutBox temp = new AboutBox(this, settings);
            AboutBox temp = new AboutBox(muckMainFrame, settings);
            temp.setVisible(true);
        }
    }
    
    private void showExtProgDialogue() {
        // ExternalProgs eP = new ExternalProgs(this, settings);
        ExternalProgs extP = new ExternalProgs(muckMainFrame, settings);
        extP.setVisible(true);
    }
    
    private void showProxyDialogue() {
        // ProxyBox pSettings = new ProxyBox(this, settings);
        ProxyBox pSettings = new ProxyBox(muckMainFrame, settings);
        // pSettings.setSize(400, 200);
        pSettings.pack();
        // pSettings.setLocation(PosTools.findCenter(this, pSettings));
        pSettings.setLocation(PosTools.findCenter(muckMainFrame, pSettings));
        pSettings.show();
    }
    
    /** Perform all the necessary steps to shut down JamochaMUD,
     * most importantly being to save the settings from this session */
    private synchronized void quitJamochaMUD() {
        // "Shut down" and save the details of our other components
        if (useSwing) {
            // Do we need to find a better location for this?  Fix Me XXX
            int dl = ((MainSwingFrame)muckMainFrame).getDividerLocation();

            settings.setJMValue(JMConfig.DIVIDERLOCATION, dl);
        }
        
        JMUD master = settings.getJMCore();
        master.quitJamochaMUD();
    }
    
    private void showPluginsDialogue() {
        // ManagePlugins plug = new ManagePlugins(this, settings);
        ManagePlugins plug = new ManagePlugins(muckMainFrame, settings);
        plug.setSize(450, 300);
        // plug.setLocation(PosTools.findCenter(this, plug));
        plug.setLocation(PosTools.findCenter(muckMainFrame, plug));
        plug.listPlugins();
        plug.show();
    }
    
    private void setAllSync(boolean state) {
        JMUD core = settings.getJMCore();
        core.setAllSync(state);
    }
    
    /** transfer focus to our data entry window, taking into consideration
     * an special events that may also need to be passed along. */
    private void transferFocus(KeyEvent event) {
        // System.err.println("Attempting to write to DataIn: " + event);
        DataIn target = settings.getDataInVariable();
        
        if (event.getKeyCode() == KeyEvent.VK_ENTER) {
            // The user hit 'ENTER', send the text out
            target.JMSendText();
        }
        
        // target.requestFocus();
        // target.JMGainFocus();
        target.jmGainFocus();
        
        if (event.getKeyCode() == KeyEvent.VK_ENTER) {
            target.JMSendText();
            event.consume();
            return;
        }
        
        if (event.getKeyCode() != KeyEvent.VK_SHIFT && event.getKeyCode() != KeyEvent.VK_CONTROL) {
            // if (!event.isControlDown() && !event.isShiftDown()) {
            if (!event.isControlDown()) {
                target.append(event.getKeyChar() + "");
                // Using keyTyped instead of keyPressed checks our "spool" condition
                target.keyTyped(event);
            }
            event.consume();
        }
    }
    
    /** Copy the selected text from our active MU
     * to the system clipboard */
    private void copyToClipboard() {
        // Clipboard c = this.getToolkit().getSystemClipboard();
        Clipboard clip = muckMainFrame.getToolkit().getSystemClipboard();
        boolean releasePause = false;
        
        try {
            StringSelection selection = new StringSelection("");
            String str = "";
            CHandler connHandler = settings.getConnectionHandler();
            
            if (useSwing) {
                anecho.gui.JMSwingText text = connHandler.getActiveMUDSwingText();
                str = text.getSelectedText();
                text.select(0, 0);
                JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
                releasePause = swingMenu.isReleasePause();
            } else {
                JMText text = connHandler.getActiveMUDText();
                str = text.getSelectedText();
                
                // After a copy we should make certain to "unhighlight" our copied area
                text.select(0, 0);
                anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
                releasePause = mainMenu.isReleasePause();
            }
            
            selection = new StringSelection(str);
            clip.setContents(selection, selection);
            
            if (releasePause) {
                pauseStatus = false;
                callSpoolText();
            }
            /*
            if (tWReleasePauseItem.getState()) {
                // Release the paused state on the JMText area if applicable
                pauseStatus = false;
                callSpoolText();        // Spool out any held text
                }
             */
        } catch (Exception except) {
            if (DEBUG) {
                System.err.println("Copy exception in MuckMain." + except);
                except.printStackTrace();
            }
        }
        
    }
    
    /** Paste the contents of our system clipboard
     * into the data-entry window */
    private void pasteFromClipboard() {
        // Clipboard c = this.getToolkit().getSystemClipboard();
        Clipboard clip = muckMainFrame.getToolkit().getSystemClipboard();
        try {
            Transferable contents = clip.getContents(this);
            String str = (String)contents.getTransferData(DataFlavor.stringFlavor);
            
            // Now we'll stick it into the dataText-thing
            DataIn dataBox = settings.getDataInVariable();
            StringBuffer buff = new StringBuffer(dataBox.getText());
            int caretPos = dataBox.getCaretPosition();
            buff.insert(caretPos, str);
            dataBox.setText(buff.toString());
        } catch (Exception except) {
            if (DEBUG) {
                System.err.println("Paste exception in MuckMain " + except);
            }
        }
    }
    
    /** Update the connection menu with the proper
     * number of MU's, indicating connected and disconnected
     * status in addition to which is our active MU*
     */
    private void updateConnectionMenu() {
        
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();
            swingMenu.updateConnectionMenu(this);
        } else {
            // muckMainFrame.setMenuBar(tWMenuBar);
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            mainMenu.updateConnectionMenu();
        }
        
        /*
        // First, check to see if we have any connections.
        // If not, then we don't need to do any more here
        CHandler connHandler = settings.getConnectionHandler();
        if (connHandler.totalConnections() < 1) {
            return;
        }
         
        // Clear the current list of MU*'s
        tWMUListMenu.removeActionListener(this);
        tWMUListMenu.removeAll();
         
        // Add the two generic entries... the next and previous choices
        StringBuffer tempName;
        MenuItem tempMenu;
        MenuShortcut shortCut;
         
        tempMenu = new MenuItem(RB("previousMU"));
        shortCut = new MenuShortcut(KeyEvent.VK_PAGE_DOWN, false);
        tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("previousMU"));
        if (connHandler.totalConnections() < 2) {
            tempMenu.setEnabled(false);
        }
        tWMUListMenu.add(tempMenu);
         
        tempMenu = new MenuItem(RB("nextMU"));
        shortCut = new MenuShortcut(KeyEvent.VK_PAGE_UP, false);
        tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("nextMU"));
        if (connHandler.totalConnections() < 2) {
            tempMenu.setEnabled(false);
        }
        tWMUListMenu.add(tempMenu);
         
        tWMUListMenu.addSeparator();
         
        int tc = connHandler.totalConnections() - 1;
        int activeConnection = connHandler.getActiveMUIndex();
         
        // Loop through our connections and build a new menu
        for (int i = 0; i <= tc; i++) {
            tempName = new StringBuffer(connHandler.getTitle(i));
            if (i == activeConnection) {
                // This is our active MU*
                // We'll just differentiate with a check beside it's name
                tWMUListMenu.add(new CheckboxMenuItem(tempName.toString(), true));
            } else {
                tempMenu = new MenuItem(tempName.toString());
                switch(i) {
                case 0:
                    shortCut = new MenuShortcut(KeyEvent.VK_1, false);
                    break;
                case 1:
                    shortCut = new MenuShortcut(KeyEvent.VK_2, false);
                    break;
                case 2:
                    shortCut = new MenuShortcut(KeyEvent.VK_3, false);
                    break;
                case 3:
                    shortCut = new MenuShortcut(KeyEvent.VK_4, false);
                    break;
                case 4:
                    shortCut = new MenuShortcut(KeyEvent.VK_5, false);
                    break;
                case 5:
                    shortCut = new MenuShortcut(KeyEvent.VK_6, false);
                    break;
                case 6:
                    shortCut = new MenuShortcut(KeyEvent.VK_7, false);
                    break;
                case 7:
                    shortCut = new MenuShortcut(KeyEvent.VK_8, false);
                    break;
                case 8:
                    shortCut = new MenuShortcut(KeyEvent.VK_9, false);
                    break;
                case 9:
                    shortCut = new MenuShortcut(KeyEvent.VK_0, false);
                    break;
                }
         
                tempMenu.setShortcut(shortCut);
         
                tempMenu.setActionCommand("ChangeMU:" + i);
                tempMenu.addActionListener(this);
                tWMUListMenu.add(tempMenu);
            }
        }
         
        tWMUListMenu.addActionListener(this);
         */
        
    }
    
    /**  Set the state of our connection menu
     * based on our active MU*'s condition
     */
    public void setConnectionMenu() {
        CHandler connection = settings.getConnectionHandler();
        
        if (connection.isActiveMUDConnected()) {
            connectMenu();
        } else {
            disconnectMenu();
        }
        
        // Update the list of active/inactive MU*s
        updateConnectionMenu();
    }
    
    /** Clear all the plugins from our current menu
     */
    public void removeAllPlugins() {
        /*
        tWPlugInMenu.removeAll();
        tWPlugInMenu.add(managePlugIn);
        tWPlugInMenu.addSeparator();
         */
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            swingMenu.removeAllPlugins();
        } else {
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            mainMenu.removeAllPlugins();
        }
        
    }
    
    /** Add a new plugin to our plugins menu
     * @param name      Name of the plug-in
     * @param state     If the plug-in has adjustable properties (true) or not (false)
     * @param active    If the plug-in is currently enabled or not
     */
    // public void addPlugin(String name, boolean state) {
    public void addPlugin(PlugInterface plug) {
        // public void addPlugin(String name, boolean state, boolean active) {
        /*
        MenuItem tempName = new MenuItem(name);
        tempName.setEnabled(state);
        // tWPlugInMenu.add(name);
        tWPlugInMenu.add(tempName);
         */
        
        // String itemName = name;
        
        /*
        if (!active) {
            itemName = name + " " + RB("notEnabled");
        }
         */
        
        if (useSwingMenu) {
            JMainMenu swingMenu = JMainMenu.getInstance();        // Trying out the JMTFKeys singleton
            // swingMenu.addPlugin(name, state);
            // swingMenu.addPlugin(itemName, state);
            swingMenu.addPlugin(plug);
        } else {
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            // mainMenu.addPlugin(name, state);
            // mainMenu.addPlugin(itemName, state);
            mainMenu.addPlugin(plug);
        }
        
    }
    
    /** Add a new MU* to our cardlayout */
    /*
    public synchronized void addNewMU(MuSocket mu) {
        String stamp = mu.getTimeStamp() + "";
        JMText text = mu.getTextWindow();
        text.addKeyListener(this);
     
        textPanel.add(text, stamp);
     
        // We'll switch focus to the last connection we've added
        textCardLayout.first(textPanel);
     
        // Make certain our connection menu is up-to-date
        updateConnectionMenu();
        checkCloseMenuState();
     
        // textPanel.invalidate();
        textPanel.validate();
        // System.err.println("MuckMain addNewMU added " + mu);
        }
     */
    
    /** This method is used to make sure all the classes
     * properly identify the active MU*
     */
    public void setActiveMU(String muName) {
        // First we take apart our String to get the MU number
        int muNum, split;
        split = muName.indexOf(':');
        split++;
        muNum = Integer.parseInt(muName.substring(split));
        
        CHandler connHandler = settings.getConnectionHandler();
        connHandler.setActiveMU(muNum);
        
        checkCloseMenuState();
        // Now make our MU visible
        // setVisibleMU();
    }
    
    // public synchronized void setMainLayout() {
    public void setMainLayout() {
        CHandler connHandler = settings.getConnectionHandler();
        
        // To test we're going to avoid the rest of the method altogether!
        // Fix Me XXX
        if (useSwing) {
            // Add the text entry area into our Split-pane
            DataIn input = settings.getDataInVariable();
            javax.swing.JScrollPane tempText = (javax.swing.JScrollPane)(input.exportText());
            ((MainSwingFrame)muckMainFrame).setPaneComponent(tempText ,MainSwingFrame.BOTTOM);
            
            // Add the main tabbed panel to our Split-pane
            javax.swing.JTabbedPane tempPane;
            Object textObj = settings.getJMObject(JMConfig.TEXTPANEL);
            
            if (DEBUG) {
                System.err.println("MuckMain.setMainLayout() successfully received our JTabbedPane.");
            }
            // System.err.println("Our Object is: " + textObj);
            // javax.swing.JTabbedPane tempPane = (javax.swing.JTabbedPane)settings.getJMObject(JMConfig.TEXTPANEL);
            
            // We'll do the lazy initialisation here, as we can't guarantee that we'll have
            // a usable JTabbedPane by the time we get here.
            if (textObj == null) {
                if (DEBUG) {
                    System.err.println("MuckMain.setMainLayout() doing lazy initialisation of JTabbedPane");
                }
                
                tempPane = new javax.swing.JTabbedPane();
                settings.setJMValue(JMConfig.TEXTPANEL, tempPane);
                
                if (DEBUG) {
                    System.err.println("MuckMain.setMainLayout() JTabbedPane successfully initialised and stored.");
                }
            } else {
                // javax.swing.JTabbedPane tempPane = (javax.swing.JTabbedPane)textObj;
                tempPane = (javax.swing.JTabbedPane)textObj;
            }
            ((MainSwingFrame)muckMainFrame).setPaneComponent(tempPane ,MainSwingFrame.TOP);
            
            int dl = settings.getJMint(JMConfig.DIVIDERLOCATION);
            
            if (dl > 0) {
                ((MainSwingFrame)muckMainFrame).setDividerLocation(settings.getJMint(JMConfig.DIVIDERLOCATION));
            }

            return;
        }
        // boolean validLayout = settings.getJMboolean(JMConfig.MAINLAYOUTVALID);
        // boolean useSwing = settings.getJMboolean(JMConfig.USESWING);
        /*
        if (settings.getJMboolean(JMConfig.MAINLAYOUTVALID)) {
            // We already have a valid layout and don't need to do this again
            return;
        }
         
        CHandler connHandler = settings.getConnectionHandler();
        if (connHandler.totalConnections() < 1) {
            // System.err.println("MuckMain.setMainLayout: No connections available, returning.");
            return;
            }
         */
        
        // If we already have a valid layout or we don't have any connections, we won't do the layout
        /*
        if (validLayout || connHandler.totalConnections() < 1) {
            System.err.println("MuckMain.setMainLayout() No connections yet.");
            System.err.println("validLayout is: " + validLayout);
            System.err.println("total connections are: " + connHandler.totalConnections());
            return;
        }
         */
        
        if (DEBUG) {
            System.err.println("MuckMain.setMainLayout...");
        }
        // this.setLayout(null);
        if (useSwing) {
            ((javax.swing.JFrame)muckMainFrame).getContentPane().setLayout(null);
        } else {
            muckMainFrame.setLayout(null);
        }
        
        // textPanel.setLayout(null);
        int style = viewStyle;
        DataIn input = settings.getDataInVariable();
        mainBagLayout = new GridBagLayout();
        constraints = new GridBagConstraints();
        // this.setLayout(mainBagLayout);
        // muckMainFrame.setLayout(mainBagLayout);
        if (useSwing) {
            ((javax.swing.JFrame)muckMainFrame).getContentPane().setLayout(mainBagLayout);
        } else {
            muckMainFrame.setLayout(mainBagLayout);
        }
        
        // textPanel.setLayout(mainBagLayout);
        
        if (style == COMBINED) {
            // System.err.println("MuckMain.setMainLayout: doing combined layout");
            // remove(textPanel);
            // get rid of the existing layout
            // constraints = new GridBagConstraints();
            // this.setLayout(mainBagLayout);
            
            // Set the style for the combined window
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 5;
            constraints.gridx = 0;
            constraints.gridy = 0;
            // constraints.weightx = 1;
            // constraints.weighty = 5;
            constraints.weightx = 1;
            constraints.weighty = 1;
            constraints.insets = new Insets(0, 0, 2, 0);
            constraints.fill = GridBagConstraints.BOTH;
            constraints.anchor = GridBagConstraints.CENTER;
            // mainBagLayout.setConstraints(connHandler.getTextObject(), constraints);  // Ugly code Fix me XXX
            // mainBagLayout.setConstraints(textPanel, constraints);
            // System.err.println("Going to add textPanel.");
            // System.err.println("Textpanel: " + textPanel);
            // this.add(textPanel);
            if (useSwing) {
                if (DEBUG) {
                    System.err.println("MuckMain.setMainLayout() trying to get JTabbedPane.");
                }
                
                javax.swing.JTabbedPane tempPane;
                Object textObj = settings.getJMObject(JMConfig.TEXTPANEL);
                
                if (DEBUG) {
                    System.err.println("MuckMain.setMainLayout() successfully received our JTabbedPane.");
                }
                // System.err.println("Our Object is: " + textObj);
                // javax.swing.JTabbedPane tempPane = (javax.swing.JTabbedPane)settings.getJMObject(JMConfig.TEXTPANEL);
                
                // We'll do the lazy initialisation here, as we can't guarantee that we'll have
                // a usable JTabbedPane by the time we get here.
                if (textObj == null) {
                    if (DEBUG) {
                        System.err.println("MuckMain.setMainLayout() doing lazy initialisation of JTabbedPane");
                    }
                    
                    tempPane = new javax.swing.JTabbedPane();
                    settings.setJMValue(JMConfig.TEXTPANEL, tempPane);
                    
                    if (DEBUG) {
                        System.err.println("MuckMain.setMainLayout() JTabbedPane successfully initialised and stored.");
                    }
                } else {
                    // javax.swing.JTabbedPane tempPane = (javax.swing.JTabbedPane)textObj;
                    tempPane = (javax.swing.JTabbedPane)textObj;
                }
                
                mainBagLayout.setConstraints(tempPane, constraints);  // Ugly code Fix me XXX
                ((javax.swing.JFrame)muckMainFrame).getContentPane().add(tempPane);
                
                if (!listenerSet) {
                    // System.err.println("Adding listener to our JTabbedPane.");
                    // javax.swing.JTabbedPane tempPanel = (javax.swing.JTabbedPane)settconnHandler.getTextObject();
                    
                    // if (DEBUG) {
                    // System.err.println("MuckMain: Change listener temporarily disabled");
                    // }
                    
                    tempPane.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent change) {
                            if (DEBUG) {
                                System.err.println("MuckMain stateChanged.  Calling checkCloseMenuState()");
                            }
                            
                            checkCloseMenuState();
                            
                            if (DEBUG) {
                                System.err.println("MuckMain stateChanged.  Calling updateConnectionMenu()");
                            }
                            
                            updateConnectionMenu();
                            // Should change Main Title here, too.  Fix Me XXX
                            if (DEBUG) {
                                System.err.println("MuckMain: Tab change event: " + change);
                            }
                            
                            javax.swing.JTabbedPane src = (javax.swing.JTabbedPane)(change.getSource());
                            
                            if (DEBUG) {
                                System.err.println("MuckMain... we now have the JTabbedPane object.");
                            }
                            
                            if (src.isValid()) {
                                // We don't want to try and set the title if it
                                // isn't yet a valid object
                                if (DEBUG) {
                                    System.err.println("Our object is valid, so we will set the window title.");
                                }
                                setWindowTitle();
                            }
                            // CHandler cHandler = settings.getConnectionHandler();
                            // int index = cHandler.getActiveMUIndex();
                            // System.err.println("MuckMain index = " + index);
                            // System.err.println("title = " + title);
                        }
                    });
                    if (DEBUG) {
                        System.err.println("MuckMain setting listenerSet to true.");
                    }
                    
                    listenerSet = true;
                    
                }
                
            } else {
                anecho.JamochaMUD.legacy.JMTabPanel tempPane;
                Object textObj = settings.getJMObject(JMConfig.TEXTPANEL);
                // anecho.JamochaMUD.legacy.JMTabPanel tempPane = (anecho.JamochaMUD.legacy.JMTabPanel)connHandler.getTextObject();
                if (textObj == null) {
                    tempPane = new anecho.JamochaMUD.legacy.JMTabPanel();
                    settings.setJMValue(JMConfig.TEXTPANEL, tempPane);
                } else {
                    // anecho.JamochaMUD.legacy.JMTabPanel tempPane = (anecho.JamochaMUD.legacy.JMTabPanel)settings.getJMObject(JMConfig.TEXTPANEL);
                    tempPane = (anecho.JamochaMUD.legacy.JMTabPanel)textObj;
                }
                
                mainBagLayout.setConstraints(tempPane, constraints);  // Ugly code Fix me XXX
                // this.add(tempPane);      // New implementation of components
                muckMainFrame.add(tempPane);
            }
            // System.err.println("textPanel added.");
            
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = GridBagConstraints.REMAINDER;
            // constraints.gridx = 1;
            constraints.gridx = 0;
            constraints.gridy = 5;
            // constraints.weightx = 1;
            // constraints.weighty = 2;
            constraints.weighty = 0;
            constraints.insets = new Insets(0, 0, 0, 0);
            constraints.fill = GridBagConstraints.BOTH;
            constraints.anchor = GridBagConstraints.CENTER;
            // TextArea tempText = in.exportText();
            
            // System.err.println("Going to add tempText");
            // this.add(tempText);
            // if (useSwing) {
            if (settings.getJMboolean(JMConfig.USESWINGENTRY)) {
                /*
                javax.swing.JTextArea tempText = (javax.swing.JTextArea)(in.exportText());
                mainBagLayout.setConstraints(tempText, constraints);
                ((javax.swing.JFrame)muckMainFrame).getContentPane().add(tempText);
                 */
                javax.swing.JScrollPane tempText = (javax.swing.JScrollPane)(input.exportText());
                mainBagLayout.setConstraints(tempText, constraints);
                ((javax.swing.JFrame)muckMainFrame).getContentPane().add(tempText);
            } else {
                TextArea tempText = (TextArea)(input.exportText());
                mainBagLayout.setConstraints(tempText, constraints);
                if (useSwing) {
                    ((javax.swing.JFrame)muckMainFrame).getContentPane().add(tempText);
                } else {
                    muckMainFrame.add(tempText);
                }
            }
            
        } else {
            // System.err.println("MuckMain.setMainLayout: doing split layout.");
            // remove(textPanel);
            // remove(tempText);
            // mainBagLayout = new GridBagLayout();
            // constraints = new GridBagConstraints();
            // this.setLayout(mainBagLayout);
            // Set the style for the separate windows
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = GridBagConstraints.REMAINDER;
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.weightx = 1;
            constraints.weighty = 1;
            constraints.insets = new Insets(0, 0, 0, 0);
            constraints.fill = GridBagConstraints.BOTH;
            constraints.anchor = GridBagConstraints.CENTER;
            mainBagLayout.setConstraints(connHandler.getTextObject(), constraints);
            // mainBagLayout.setConstraints(textPanel, constraints);
            // add(textPanel);
            // this.add(connHandler.getTextObject());  // New version of component
            if (useSwing) {
                ((javax.swing.JFrame)muckMainFrame).getContentPane().add(connHandler.getTextObject());
            } else {
                muckMainFrame.add(connHandler.getTextObject());
            }
            
            // Now make our input bar visible again
            // DataIn in = settings.getDataInVariable();
            input.restoreText();
            
        }
        
        // mainBagLayout.invalidateLayout(this);
        // mainBagLayout.layoutContainer(this);
        mainBagLayout.invalidateLayout(muckMainFrame);
        mainBagLayout.layoutContainer(muckMainFrame);
        muckMainFrame.invalidate();
        muckMainFrame.validate();
        muckMainFrame.doLayout();
        
        /*
        this.invalidate();
        this.validate();
        this.doLayout();
         */
        
        // Should we try and absolute size the DataIn?
        // Fix this XXX - in the future we want the user to decide how many
        // lines tall our DataIn textarea is!
        input.setRows(3);
        
        viewStyle = style;
        
        // System.err.print("MuckMain.setMainLayout: going to updateConnectionMenu...");
        updateConnectionMenu();
        // System.err.println("complete.");
        settings.setJMValue(JMConfig.MAINLAYOUTVALID, true);
        
    }
    
    /** Remove the active view from our list of MU*s.  Since some Operating Systems
     * seem to synchronize things in ways that I hadn't foresee previously, we'll
     * have to do everything by the time-stamp of the MU, so we don't mess things up!
     */
    private void closeActiveWindow() {
        // remove the MU from our Cardlayout
        CHandler connHandler = settings.getConnectionHandler();
        // System.err.println("MuckMain.closeActiveWindow: calling connHandler.getActiveWindow()");
        MuSocket muSock = connHandler.getActiveMUHandle();
        // JMText text = connHandler.getActiveMUDText();
        // String stamp = muSock.getTimeStamp() + "";
        
        /*
        // As a cosmetic concern, lets turn this component black first
        // textCardLayout.removeLayoutComponent(text);
         
        // If we are still connected to this MU*, run the disconnect
        // if (connHandler.isActiveMUDConnected()) {
        // connHandler.closeActiveMU(stamp);
        // }
         
        // remove the MU* from our layout manager
        textPanel.remove(text);
         
        // update our CHandler
        connHandler.removeActiveMU(stamp);
         
        // Now we should change our view to the next active MU*
        setWindowTitle();
        setVisibleMU();
         
        checkCloseMenuState();
         */
        
        closeWindow(muSock);
        // connHandler.removeActiveMU(stamp);
    }
    
    /** Close a window based on the passed MuSocket */
    private void closeWindow(MuSocket muSock) {
        CHandler connHandler = settings.getConnectionHandler();
        // MuSocket muSock = connHandler.getActiveMUHandle();
        // JMText text = connHandler.getActiveMUDText();
        // String stamp = muSock.getTimeStamp() + "";
        // JMText text = connHandler.getActiveMUDText(stamp);
        // textPanel.removeLayoutComponent(text);
        // textCardLayout.removeLayoutComponent(text);
        // textCardLayout.invalidateLayout(this);
        
        connHandler.removeMU(muSock);  // Causes layout problems somewhere down the line XXX
        
        // setWindowTitle();
        // setVisibleMU();
        
        // checkCloseMenuState();
        updateStates();
    }
    
    /** Update the window title, set the visible MU* and make
     * certain that our connection menu is showing the proper states
     */
    private void updateStates() {
        setWindowTitle();
        setVisibleMU();
        
        checkCloseMenuState();
    }
    
    /*
    private void checkPause() {
    // CHandler target = settings.getConnectionHandler();
    // MuSocket mSock = target.getActiveMUHandle();
    // if (mSock.isPaused()) {
    // This spools out any paused text
    // mSock.SpoolText();
    // }
    callSpoolText();
    }
     */
    
    
    /** This method checks to see if the &quot;Close current view"
     * Menu item should be active or not
     */
    private void checkCloseMenuState() {
        // private synchronized void checkCloseMenuState() { // Synchronized causes **HANGS!!**
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState()");
        }
        
        CHandler connHandler = settings.getConnectionHandler();
        
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState() got our connHandler: " + connHandler);
        }
        
        /*
        if (connHandler.totalConnections() > 1) {
            closeMU.setEnabled(true);
        } else {
            closeMU.setEnabled(false);
            }
         */
        int totalConn = 0;
        boolean closeEnabled = false;
        boolean connectEnabled = false;
        
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState() getting total connections.");
        }
        
        totalConn = connHandler.totalConnections();
        
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState() total connections: " + totalConn);
        }
        
        if (connHandler.isActiveMUDConnected()) {
            connectEnabled = true;
        }
        
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState() activeMUDConnected set to: " + connectEnabled);
        }
        
        if (useSwingMenu) {
            if (DEBUG) {
                System.err.println("MuckMain.checkCloseMenuState() working with Swing Menu.");
            }
            
            JMainMenu swingMenu = JMainMenu.getInstance();
            
            // swingMenu.buildMenu((javax.swing.JFrame)muckMainFrame, this);
            // if (connHandler.totalConnections() > 1) {
            if (totalConn > 1) {
                // swingMenu.setCloseMUEnabled(true);
                closeEnabled = true;
            }
            // else {
            // swingMenu.setCloseMUEnabled(false);
            // }
            swingMenu.setCloseMUEnabled(closeEnabled);
            swingMenu.setConnected(connectEnabled);
            
        } else {
            if (DEBUG) {
                System.err.println("MuckMain.checkCloseMenuState() working with AWT Menu.");
            }
            anecho.JamochaMUD.legacy.MainMenu mainMenu = anecho.JamochaMUD.legacy.MainMenu.getInstance();
            
            if (DEBUG) {
                System.err.println("MuckMain.checkCloseMenuState() checking totalConnections.");
            }
            
            // if (connHandler.totalConnections() > 1) {
            if (totalConn > 1) {
                if (DEBUG) {
                    System.err.println("MuckMain.checkCloseMenuState() more than one connection.");
                }
                // mainMenu.setCloseMUEnabled(true);
                closeEnabled = true;
            } else {
                if (DEBUG) {
                    System.err.println("MuckMain.checkCloseMenuState() total connections 1 or less.");
                }
            }
            //else {
            //mainMenu.setCloseMUEnabled(false);
            // }
            if (DEBUG) {
                System.err.println("MuckMain.checkCloseMenuState() setCloseMUEnabled to: " + closeEnabled);
            }
            
            mainMenu.setCloseMUEnabled(closeEnabled);
            
            if (DEBUG) {
                System.err.println("MuckMain.checkCloseMenuState() setConnected to: " + connectEnabled);
            }
            mainMenu.setConnected(connectEnabled);
            // mainMenu.buildMenu(muckMainFrame, this, settings);
        }
        
        if (DEBUG) {
            System.err.println("MuckMain.checkCloseMenuState() completed.");
        }
        
    }
    
    /** display the next or previous MU, depending on
     * the direction given
     */
    private void advanceMU(int direction) {
        CHandler handler = settings.getConnectionHandler();
        
        // change the active MU*
        if (direction == NEXT) {
            handler.nextMU();
        } else {
            handler.previousMU();
        }
        
        // now update our display
        setWindowTitle();
        setVisibleMU();
    }
    
    private void setDoubleBuffer(boolean state) {
        // settings.setDoubleBuffer(state);
        settings.setJMValue(JMConfig.DOUBLEBUFFER, state);
        CHandler handler = settings.getConnectionHandler();
        handler.setDoubleBuffer(state);
    }
    
    private void setAntiAliasing(boolean state) {
        if (DEBUG) {
            System.err.println("MuckMain.setAntiAliasing to : " + state);
        }
        settings.setJMValue(JMConfig.ANTIALIAS, state);
        CHandler handler = settings.getConnectionHandler();
        handler.setAntiAliasing(state);
        
        // Change our text entry area
        settings.getDataInVariable().setAntiAliasing(state);
    }
    
    private void setLowColour(boolean state) {
        if (DEBUG) {
            System.err.println("MuckMain settings LOWCOLOUR to: " + state);
        }
        settings.setJMValue(JMConfig.LOWCOLOUR, state);
        CHandler handler = settings.getConnectionHandler();
        handler.setLowColour(state);
    }
    
    /** Call the SpoolText proceedure that belongs to the correct JMText */
    private void callSpoolText() {
        CHandler target = settings.getConnectionHandler();
        
        // Check to see if we actually have any connections before we try to spool text
        if (target.totalConnections() > 0) {
            
            // System.err.println("MuckMain.callSpoolText calls getActiveMUHandle");
            MuSocket mSock = target.getActiveMUHandle();
            if (mSock.isPaused()) {
                // System.err.println("Window deiconified shows mSock is paused...");
                // This spools out any paused text
                mSock.spoolText();
            }
        }
    }
    
    private void installPlugIn() {
        EnumPlugIns plugEnum = EnumPlugIns.getInstance(settings);
        plugEnum.addNewPlugIn();
    }
    
    /** Rebuild the plug-in menu to reflect its current state.
     * This is usually called after activating/deactivating plug-ins
     */
    public void rebuildPlugInMenu() {
        if (DEBUG) {
            System.err.println("MuckMain.rebuildPlugInMenu() called.");
        }
        
        // Clear out all the old entries
        removeAllPlugins();
        
        // Cycle through our list of plug-ins and add them back to the menu
        Vector pClasses = EnumPlugIns.plugInClass;
        
        /*
        int selected = EnumPlugIns.plugInName.indexOf(testName);
        Object plugClass = EnumPlugIns.plugInClass.elementAt(selected);
        String name = ((PlugInterface)plugClass).PlugInName();
         */
        
        int pcs = pClasses.size();
        
        for (int i = 0; i < pcs; i++) {
            addPlugin((PlugInterface)pClasses.elementAt(i));
        }
        
    }
}